----------------------------------------------------------------------------------------------------
local msg = function(M) reaper.ShowConsoleMsg(tostring(M).."\n") end
----------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------
local utf8_decode={
  [128]={[147]='\150',[148]='\151',[152]='\145',[153]='\146',[154]='\130',[156]='\147',[157]='\148',[158]='\132',[160]='\134',[161]='\135',[162]='\149',[166]='\133',[176]='\137',[185]='\139',[186]='\155'},
  [130]={[172]='\136'},
  [132]={[150]='\185',[162]='\153'},
  [194]={[152]='\152',[160]='\160',[164]='\164',[166]='\166',[167]='\167',[169]='\169',[171]='\171',[172]='\172',[173]='\173',[174]='\174',[176]='\176',[177]='\177',[181]='\181',[182]='\182',[183]='\183',[187]='\187'},
  [208]={[129]='\168',[130]='\128',[131]='\129',[132]='\170',[133]='\189',[134]='\178',[135]='\175',[136]='\163',[137]='\138',[138]='\140',[139]='\142',[140]='\141',[143]='\143',[144]='\192',[145]='\193',[146]='\194',[147]='\195',[148]='\196',
  [149]='\197',[150]='\198',[151]='\199',[152]='\200',[153]='\201',[154]='\202',[155]='\203',[156]='\204',[157]='\205',[158]='\206',[159]='\207',[160]='\208',[161]='\209',[162]='\210',[163]='\211',[164]='\212',[165]='\213',[166]='\214',
  [167]='\215',[168]='\216',[169]='\217',[170]='\218',[171]='\219',[172]='\220',[173]='\221',[174]='\222',[175]='\223',[176]='\224',[177]='\225',[178]='\226',[179]='\227',[180]='\228',[181]='\229',[182]='\230',[183]='\231',[184]='\232',
  [185]='\233',[186]='\234',[187]='\235',[188]='\236',[189]='\237',[190]='\238',[191]='\239'},
  [209]={[128]='\240',[129]='\241',[130]='\242',[131]='\243',[132]='\244',[133]='\245',[134]='\246',[135]='\247',[136]='\248',[137]='\249',[138]='\250',[139]='\251',[140]='\252',[141]='\253',[142]='\254',[143]='\255',[144]='\161',[145]='\184',
  [146]='\144',[147]='\131',[148]='\186',[149]='\190',[150]='\179',[151]='\191',[152]='\188',[153]='\154',[154]='\156',[155]='\158',[156]='\157',[158]='\162',[159]='\159'},[210]={[144]='\165',[145]='\180'}
}

local nmdc = {
  [36] = '$',
  [124] = '|'
}

function Utf8ToAnsi(s)
  local a, j, r, b = 0, 0, ''
  for i = 1, s and s:len() or 0 do
    b = s:byte(i)
    if b < 128 then
      if nmdc[b] then r = r..nmdc[b]
      else r = r..string.char(b) end
    elseif a == 2 then
      a, j = a - 1, b
    elseif a == 1 then
      a, r = a - 1, r..utf8_decode[j][b]
    elseif b == 226 then
      a = 2
    elseif b == 194 or b == 208 or b == 209 or b == 210 then
      j, a = b, 1
    else r = r..'_' end
  end
  return r
end

----------------------------------------------------------------------------------------------------
-- Load some modules -----------------
local File = require "Modules.File"
--******************************************************************************
--------------------------------------
function SetRGB(RGB, a)
  local r = (RGB & 0xFF0000) / 16777216;
  local g = (RGB & 0x00FF00) / 65536;
  local b = (RGB & 0x0000FF) / 256;
  gfx.set(r,g,b, a or 1);
end

--********************************************************************
local Presets = {}

-- устан. пресет-папку(fullpath), создает если не существует
function Presets.SetPath(path)
  if File.RecursiveCreateDirectory(path) then
    Presets.path = path
    return true
  end
end

-- устан. расш. для валидных файлов пресетов
-- ext - непустая строка, например, "txt", "mpr", без точек!
-- лучше пробить по допустимым символам, для себя и так норм
function Presets.SetExtension(ext)
  if Presets.ext == "" then return false
  else Presets.ext = ext return true
  end
end

-- создает список существующих пресетов
function Presets.UpdateList()
  if not Presets.path then return end
  local t = File.EnumerateFiles(Presets.path)
  local ptn = "(.+)%." .. Presets.ext .. "$" -- паттерн для поиска
  Presets.List = {}
  if t then
    for i = 1, #t do
      local name = t[i]:match(ptn) -- пресет-файлы, захв. имя
      if name then Presets.List[#Presets.List+1] = name end
    end
  end
  return Presets.List
end

-- устан.(не загружает!) текущий пресет(имя, отображаемое в пресет-строке)
function Presets.SetCurPresetName(curpresetname)
  if curpresetname and type(curpresetname) == "string" and #curpresetname > 0 then
    Presets.curpresetname = curpresetname
  else Presets.curpresetname = "Concert Preset"
  end
end

-- возвр. индекс пресета по имени(либо nil)
function Presets.GetByName(presetname)
  if not Presets.List then return end
  for k, v in pairs(Presets.List) do
    if v == presetname then return k end
  end
end

-- возвр. имя пресета по индекс(либо nil)
function Presets.GetName(presetidx)
  if not Presets.List then return end
  return Presets.List[presetidx]
end

-- читает и возвращает данные из файла пресета
function Presets.Load(presetname)
  local filename = Presets.path .. "/" .. presetname .. "." .. Presets.ext
  return File.ReadBin(filename) -- ret data if succefully or nil
end

-- сохр. пресет в пресетную папку с соотв. расширением
function Presets.Save(presetname, presetdata)
  local presetname = Utf8ToAnsi(presetname)
  local filename = Presets.path .. "/" .. presetname .. "." .. Presets.ext
  return File.WriteBin(filename, presetdata) -- ret true if succefully
end

-- переименовывает пресет oldname > newname
function Presets.Rename(oldname, newname)
  local oldname =  Utf8ToAnsi(oldname)
  local newname =  Utf8ToAnsi(newname)
  oldname = Presets.path .. "/" .. oldname .. "." .. Presets.ext
  newname = Presets.path .. "/" .. newname .. "." .. Presets.ext
  return File.Rename(oldname, newname)
end

-- удаляет файл пресета
function Presets.Delete(presetname)
  local presetname = Utf8ToAnsi(presetname)
  local filename = Presets.path .. "/" .. presetname .. "." .. Presets.ext

  -- msg(filename)
  return File.Remove(filename) -- ret true if succefully
end

------------------------------------------------------------
-- Init Preset Menu ----------------------------------------
------------------------------------------------------------
function Presets.MenuInit(x,y,w,h, col, col_font, path, ext, f_s)
  if Presets.SetPath(path) and
    Presets.SetExtension(ext) and
    Presets.UpdateList() then
    Presets.Menu = {x = x, y = y, w = w, h = h, col = col, col_font = col_font, font_s = f_s}
    Presets.SetCurPresetName() -- now nil, "No Preset"
    return true -- if succefully
  end
end

------------------------------------------------------------
-- Verify and confirm Preset Menu actions ------------------
------------------------------------------------------------
-- Можно поразносить по соотв. функциям проверку и подтверждение
-- Это, наверное, удобнее и проще.
function Presets.ConfirmAction(action, name)
  ------------------
  if action == "Load" then
    if not Presets.GetByName(name) then return end
  end
  ------------------
  if action == "Save" then
    if name ~= Presets.curpresetname and Presets.GetByName(name) then
      local msg = "Overwrite \"" .. name .. "\"?"
      if reaper.MB(msg, "Save Preset", 1) ~= 1 then return end
    end
  end
  ------------------
  if action == "Rename" then
    if not Presets.GetByName(Presets.curpresetname) then return end
    if name ~= Presets.curpresetname and Presets.GetByName(name) then
      local msg = "Overwrite \"" .. name .. "\"?"
      if reaper.MB(msg, "Rename Preset", 1) ~= 1 then return end
      action = "Save" -- It's Overwrite Exist Preset!
    end
  end
  ------------------
  if action == "Delete" then
    if not Presets.GetByName(name) then return end
    local msg = "Delete preset \"" .. name .. "\"?"
    if reaper.MB(msg, "Confirm Preset Delete", 1) ~= 1 then return end
  end
  ------------------
  return action, name
end

------------------------------------------------------------
-- Draw Preset Menu ----------------------------------------
------------------------------------------------------------
function Presets.MenuDraw()
  if not Presets.List then return end
  local x,y,w,h = Presets.Menu.x, Presets.Menu.y, Presets.Menu.w, Presets.Menu.h
  local col, col_font, font_s = Presets.Menu.col, Presets.Menu.col_font, Presets.Menu.font_s
  -- coords --------
  local x1,y1,w1,h1 = x,y,h,h    -- prev preset
  local x2,y2,w2,h2 = x+h+2,y,w-h*3-6, h -- curpreset
  local x3,y3,w3,h3 = x+w-h*2-2, y, h, h -- next preset
  local x4,y4,w4,h4 = x+w-h, y, h, h -- menu btn

  gfx.setfont(1,"Verdana", font_s) -- presets font

  SetRGB(col, 0.03) -- bg color
  gfx.rect(x,y,w,h, 1)  -- main rect
  -- gfx.set(0.5, 0.5, 0.5, 0.15) -- frames col
  SetRGB(col,0.4)
  gfx.rect(x1,y1,w1,h1,0) -- prev rect
  gfx.rect(x2,y2,w2,h2,0) -- cur rect
  gfx.rect(x3,y3,w3,h3,0) -- next rect
  gfx.rect(x4,y4,w4,h4,0) -- menu rect

  SetRGB(col_font, 0.8)
  gfx.x = x1; gfx.y = y1
  gfx.drawstr("<", 5, x1+w1, y1+h1) -- prev "<"
  gfx.x = x2; gfx.y = y2
  gfx.drawstr(Presets.curpresetname, 5, x2+w2, y2+h2) -- currrent
  gfx.x = x3; gfx.y = y3
  gfx.drawstr(">", 5, x3+w3, y4+h3) -- next ">"
  gfx.x = x4; gfx.y = y4
  gfx.drawstr("+", 5, x4+w4, y4+h4) -- menu "+"

  --------------------------------------
  local action, name -- init state!

  -- Menu List ---------------
  if mouseDown(x2,y2,w2,h2) then -- mouse down in curpreset rect
    gfx.x = x2; gfx.y = y2+h2;
    local menu_ret = gfx.showmenu(table.concat(Presets.List,"|"))
    if menu_ret > 0 then action, name = "Load", Presets.List[menu_ret] end -- Load
  end
  -- Prev Preset -------------
  if mouseDown(x1,y1,w1,h1) then -- mouse down in prev("<") rect
    local idx = Presets.GetByName(Presets.curpresetname)
    if not idx or idx == 1 then idx = #Presets.List+1 end
    action, name = "Load", Presets.List[idx-1] -- Load Prev
  end
  -- Next Preset -------------
  if mouseDown(x3,y3,w3,h3) then -- mouse down in next(">") rect
    local idx = Presets.GetByName(Presets.curpresetname)
    if not idx or idx == #Presets.List then idx = 0 end
    action, name = "Load", Presets.List[idx+1] -- Load Next
  end
  -- Menu Button -------------
  if mouseDown(x4,y4,w4,h4) then -- mouse down in menu("+") rect
    gfx.x = x4; gfx.y = y4+h4;
    local menu_ret, ui_ret
    menu_ret = gfx.showmenu("Save Preset|Rename Preset|Delete Preset")
    if menu_ret == 1 then      -- Save
      ui_ret, name = reaper.GetUserInputs("Save Preset", 1, "Preset name:,extrawidth=200", Presets.curpresetname)
      if ui_ret then action = "Save" end
    elseif menu_ret == 2 then  -- Rename
      ui_ret, name = reaper.GetUserInputs("Rename Preset", 1, "Preset name:,extrawidth=200", Presets.curpresetname)
      if ui_ret then action = "Rename" end
    elseif menu_ret == 3 then  -- Delete
      action, name = "Delete", Presets.curpresetname
    end
  end

  -- Confirm and return action, name ---
  if action and name and name ~= "" then
    return Presets.ConfirmAction(action, name)
  end

end

--================================================
return Presets
